/* 
 *     ********************************************************************* 
 *     * Copyright (C) 1988, 1990 Stanford University.                     * 
 *     * Permission to use, copy, modify, and distribute this              * 
 *     * software and its documentation for any purpose and without        * 
 *     * fee is hereby granted, provided that the above copyright          * 
 *     * notice appear in all copies.  Stanford University                 * 
 *     * makes no representations about the suitability of this            * 
 *     * software for any purpose.  It is provided "as is" without         * 
 *     * express or implied warranty.  Export of this software outside     * 
 *     * of the United States of America may require an export license.    * 
 *     ********************************************************************* 
 */

/*--------------------------------------------------------*/
/* tkana.c --- Tk/Tcl-based version of the IRSIM analyzer */
/*--------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include <tcl.h>
#include <tk.h> 

#ifdef BLT
#ifdef HAVE_BLT
#include <blt.h>
#else
#include "blt.h"
#endif
#endif

#include "defs.h"
#include "net.h"

/*------------------------------------------------------*/
extern Tcl_Interp *consoleinterp;
extern Tcl_Interp *irsiminterp;

/*------------------------------------------------------*/
extern void lprintf();

/*------------------------------------------------------*/
typedef long TimeType;

/* Structure for linked list of BLT vectors */

typedef struct _vectorlist {
   Blt_Vector *vptr;		/* The BLT vector representing the node values */
   Blt_Vector *tptr;		/* The BLT vector representing the time values */
   nptr nd;			/* Corresponding IRSIM node */
   bptr vec;			/* Corresponding IRSIM vector */
   int offset;			/* Offset value to add to BLT vector values */
   struct _vectorlist *vnext;	/* Next entry in linked list */
} vectorlist;    

vectorlist *VectorList;		/* The list of Blt vectors */

/*------------------------------------------------------*/
/* OffsetNode()						*/
/*------------------------------------------------------*/

int OffsetNode(nptr nd, int *flag)
{
    vectorlist *thisv;

    /* Find vector corresponding to this node */

    for (thisv = VectorList; thisv != NULL; thisv = thisv->vnext)
	if (thisv->nd == nd)
	    thisv->offset = *flag;

    return(1);
}

/*------------------------------------------------------*/
/* OffsetVector()					*/
/*------------------------------------------------------*/

int OffsetVector(bptr vec, int *flag)
{
    /* To be done. . . */
    return(0);
}

/*------------------------------------------------------*/
/* AddNode()						*/
/*------------------------------------------------------*/

int AddNode(nptr nd, int *flag)
{
    Blt_Vector *newbltvector, *newtimevector;
    vectorlist *newvector;
    int result;
    char *tname;

    tname = Tcl_Alloc(strlen(nd->nname) + 6);
    strcpy(tname, nd->nname);
    strcat(tname, "_time");
    result = Blt_CreateVector(irsiminterp, nd->nname, 0, &newbltvector);
    result = Blt_CreateVector(irsiminterp, tname, 0, &newtimevector);
    newvector = (vectorlist *)Tcl_Alloc(sizeof(vectorlist));

    newvector->nd = nd;
    newvector->vec = NULL;
    newvector->vptr = newbltvector;
    newvector->tptr = newtimevector;
    newvector->vnext = VectorList;
    newvector->offset = 0;
    VectorList = newvector;

    Tcl_Free(tname);
    return (1);
}

/*------------------------------------------------------*/
/* AddVector()						*/
/*------------------------------------------------------*/

int AddVector(bptr vec, int *flag)
{
    Blt_Vector *newbltvector, *newtimevector;
    vectorlist *newvector;
    int result;
    char *tname;

    tname = Tcl_Alloc(strlen(vec->name) + 6);
    strcpy(tname, vec->name);
    strcat(tname, "_time");

    result = Blt_CreateVector(irsiminterp, vec->name, 0, &newbltvector);
    result = Blt_CreateVector(irsiminterp, tname, 0, &newtimevector);
    newvector = (vectorlist *)Tcl_Alloc(sizeof(vectorlist));

    newvector->nd = NULL;
    newvector->vec = vec;
    newvector->vptr = newbltvector;
    newvector->tptr = newtimevector;
    newvector->vnext = VectorList;
    VectorList = newvector;

    Tcl_Free(tname);
    return (1);
}

/*------------------------------------------------------*/
/* DisplayTraces()					*/
/*------------------------------------------------------*/

void DisplayTraces(int isMapped)
{
}

/*------------------------------------------------------*/
/* updateTraces() ---					*/
/*							*/
/*	Local function which updates any registered	*/
/*	BLT vectors with current trace information.	*/
/*------------------------------------------------------*/

void updateTraces(TimeType endT)
{
    Blt_Vector *thisbltv;
    Blt_Vector *thisbltt;
    vectorlist *thisv;
    HistEnt *h, *hmark;
    double *xdata, *ydata, *xptr, *yptr, mapval, doff;
    int i, asize, aidx;

    /* Go through list of vectors and update the Blt vectors to */
    /* represent the current state.				*/

    for (thisv = VectorList; thisv != NULL; thisv = thisv->vnext)
    {
	thisbltv = thisv->vptr;
	thisbltt = thisv->tptr;
	doff = (double)thisv->offset;

	if (thisv->vec == NULL)
	{
	    asize = thisbltv->numValues;
	    if (asize > 0) asize--;

	    aidx = 0;
	    hmark = &(thisv->nd->head);
	    while (aidx != asize)
	    {
	       if (!hmark->punt) aidx++;
	       hmark = hmark->next;
	    }

	    h = hmark;
	    while (h != NULL && h->time <= endT)
	    {
		if (!h->punt) asize++;
		h = h->next;
	    }
	    Blt_ResizeVector(thisbltv, asize + 1);
	    Blt_ResizeVector(thisbltt, asize + 1);

	    xdata = thisbltt->valueArr;
	    ydata = thisbltv->valueArr;
	    xptr = xdata + aidx;
	    yptr = ydata + aidx;

	    h = hmark;
	    while (h != NULL && h->time <= endT)
	    {
		/* temporary */
		if ((int)(xptr - xdata) >= asize)
		    lprintf(stderr, "   Warning: exceeded expected vector length\n"); 

		if (!h->punt) {
		   char hch;
		   *xptr++ = (double)h->time;
		   switch(h->val) {
		       case HIGH:
			  mapval = doff + 1.0; hch='H'; break;
		       case LOW:
			  mapval = doff + 0.0; hch='L'; break;
		       case X:
			  mapval = doff + 0.5; hch='X'; break;
		       default:
			  mapval = doff - 1.0; hch='U'; break; /* should not happen? */
		   }
		   *yptr++ = mapval;
		}
		h = h->next;
	    }

	    /* Extend final bit value out to current end time */
	    if (yptr > ydata)
	    {
		*xptr = (double)endT;
		*yptr = *(yptr - 1);
	    }

	    Blt_ResetVector(thisbltv, ydata, thisbltv->numValues,
			thisbltv->arraySize, TCL_DYNAMIC);
	    Blt_ResetVector(thisbltt, xdata, thisbltt->numValues,
			thisbltt->arraySize, TCL_DYNAMIC);
	}
	else	/* (to be completed) */
	{
	    for (i = 0; i < thisv->vec->nbits; i++)
	    {
		h = &(thisv->vec->nodes[i]->head);
		while (h != NULL && h->time <= endT)
		{
		    h = h->next;
		}
	    }
	}
    }
}

/*------------------------------------------------------*/
/* StopAnalyzer()					*/
/*------------------------------------------------------*/

void StopAnalyzer()
{
    lprintf(stderr, "StopAnalyzer() called\n");
}

/*------------------------------------------------------*/
/* RestartAnalyzer()					*/
/*------------------------------------------------------*/

void RestartAnalyzer(long first_time, long last_time, int same_hist)
{
    lprintf(stderr, "RestartAnalyzer(%d, %d, %d) called\n",
		first_time, last_time, same_hist);
}

/*------------------------------------------------------*/
/* ClearTraces()					*/
/*------------------------------------------------------*/

void ClearTraces()
{
}

/*------------------------------------------------------*/
/* RemoveVector()					*/
/*------------------------------------------------------*/

void RemoveVector(bptr b)
{
}

/*------------------------------------------------------*/
/* RemoveNode()						*/
/*------------------------------------------------------*/

void RemoveNode(nptr n)
{
}

/*------------------------------------------------------*/
/* RemoveAllDeleted()					*/
/*------------------------------------------------------*/

void RemoveAllDeleted()
{
}

/*------------------------------------------------------*/
/* UpdateWindow()					*/
/*------------------------------------------------------*/

void UpdateWindow(TimeType endT)
{
    lprintf(stderr, "UpdateWindow(%d) called\n", endT);
    updateTraces(endT);
}

/*------------------------------------------------------*/
/* TerminateAnalyzer()					*/
/*------------------------------------------------------*/

void TerminateAnalyzer()
{
}

/*------------------------------------------------------*/
/* InitDisplay()					*/
/*------------------------------------------------------*/

int InitDisplay(char *fname, char *display_unit)
{
    /* To do:  check for presence of Tk main window;    */
    /* Return FALSE if one doesn't exist.		*/
    lprintf(stderr, "InitDisplay(%s, %s) called\n", fname, display_unit);
    return (TRUE);
}

/*------------------------------------------------------*/
/* InitTimes()						*/
/*------------------------------------------------------*/

void InitTimes(TimeType firstT, TimeType stepsize, TimeType lastT)
{
    lprintf(stderr, "InitTimes(%d, %d, %d) called\n", firstT, stepsize, lastT);
}

/*------------------------------------------------------*/
